# autogenerated by ssc-gen DO NOT_EDIT
"""Dummy parser config for http://books.toscrape.com/"""

from __future__ import annotations
import re
from typing import List, Dict, TypedDict, Union
from contextlib import suppress
import sys

if sys.version_info >= (3, 10):
    from types import NoneType
else:
    NoneType = type(None)

from parsel import Selector, SelectorList
from parsel.selector import _SelectorType  # noqa

T_Urls = List[str]
T_UrlsMap = Dict[str, str]
T_Books = TypedDict(
    "T_Books",
    {
        "name": str,
        "image_url": str,
        "url": str,
        "rating": str,
        "price": int,
    },
)
T_CataloguePage = TypedDict(
    "T_CataloguePage",
    {
        "title": str,
        "urls": T_Urls,
        "urls_map": T_UrlsMap,
        "books": List[T_Books],
    },
)


class Urls:
    """fetch add patches and urls from <a> tag

    [
        "String",
        "..."
    ]"""

    def __init__(self, document: Union[str, _SelectorType]) -> None:
        self._doc = (
            Selector(document) if isinstance(document, str) else document
        )

    def _split_doc(self, value: _SelectorType) -> SelectorList:
        return value.css("a")

    def _parse_item(self, value: Selector) -> str:
        return value.attrib["href"]

    def parse(self) -> T_Urls:
        return [self._parse_item(e) for e in self._split_doc(self._doc)]


class UrlsMap:
    """

    {
        "<K>": "String",
        "<KN>": "..."
    }"""

    def __init__(self, document: Union[str, _SelectorType]) -> None:
        self._doc = (
            Selector(document) if isinstance(document, str) else document
        )

    def _split_doc(self, value: _SelectorType) -> SelectorList:
        return value.css("a")

    def _parse_key(self, value: Selector) -> str:
        return value.attrib["href"]

    def _parse_value(self, value: Selector) -> str:
        value1 = value.get()
        return value1.strip(" ")

    def parse(self) -> T_UrlsMap:
        return {
            self._parse_key(e): self._parse_value(e)
            for e in self._split_doc(self._doc)
        }


class Books:
    """

    [
        {
            "name": "String",
            "image_url": "String",
            "url": "String",
            "rating": "String",
            "price": "Int"
        },
        "..."
    ]"""

    def __init__(self, document: Union[str, _SelectorType]) -> None:
        self._doc = (
            Selector(document) if isinstance(document, str) else document
        )

    def _split_doc(self, value: _SelectorType) -> SelectorList:
        return value.css(".col-lg-3")

    def _parse_name(self, value: Selector) -> str:
        value1 = value.css(".thumbnail")
        return value1.attrib["alt"]

    def _parse_image_url(self, value: Selector) -> str:
        value1 = value.css(".thumbnail")
        value2 = value1.attrib["src"]
        return f"https://{value2}" if value2 else value2

    def _parse_url(self, value: Selector) -> str:
        value1 = value.css(".image_container > a")
        return value1.attrib["href"]

    def _parse_rating(self, value: Selector) -> str:
        value1 = value.css(".star-rating")
        value2 = value1.attrib["class"]
        return value2.lstrip("star-rating ")

    def _parse_price(self, value: Selector) -> int:
        value1 = value
        with suppress(Exception):
            value2 = value1.css(".price_color")
            value3 = "".join(value2.css("::text").getall())
            value4 = re.search("(\\d+)", value3)[1]
            return int(value4)
        return 0

    def parse(self) -> List[T_Books]:
        return [
            {
                "name": self._parse_name(e),
                "image_url": self._parse_image_url(e),
                "url": self._parse_url(e),
                "rating": self._parse_rating(e),
                "price": self._parse_price(e),
            }
            for e in self._split_doc(self._doc)
        ]


class CataloguePage:
    """books.toscrape.com catalogue page entrypoint parser

    USAGE:

        1. GET <catalog page> (https://books.toscrape.com/, https://books.toscrape.com/catalogue/page-2.html, ...)
        2. add another prepare instruction how to correct cook page (if needed?)

    ISSUES:

        1. nope! Their love being scraped!


    {
        "title": "String",
        "urls": [
            "String",
            "..."
        ],
        "urls_map": {
            "<K>": "String",
            "<KN>": "..."
        },
        "books": [
            {
                "name": "String",
                "image_url": "String",
                "url": "String",
                "rating": "String",
                "price": "Int"
            },
            "..."
        ]
    }"""

    def __init__(self, document: Union[str, _SelectorType]) -> None:
        self._doc = (
            Selector(document) if isinstance(document, str) else document
        )

    def _pre_validate(self, value: Union[_SelectorType]) -> None:
        value1 = value.css("title")
        value2 = "".join(value1.css("::text").getall())
        assert re.search("Books to Scrape", value2), ""
        return

    def _parse_title(self, value: Selector) -> str:
        value1 = value
        with suppress(Exception):
            assert value1.css("title"), ""
            value2 = value1
            value3 = value2.css("title")
            value4 = "".join(value3.css("::text").getall())
            value5 = re.sub("^\\s+", "", value4)
            return re.sub("\s+$", "", value5)
        return "test"

    def _parse_urls(self, value: Selector) -> T_Urls:
        return Urls(value).parse()

    def _parse_urls_map(self, value: Selector) -> T_UrlsMap:
        return UrlsMap(value).parse()

    def _parse_books(self, value: Selector) -> List[T_Books]:
        return Books(value).parse()

    def parse(self) -> T_CataloguePage:
        self._pre_validate(self._doc)
        return {
            "title": self._parse_title(self._doc),
            "urls": self._parse_urls(self._doc),
            "urls_map": self._parse_urls_map(self._doc),
            "books": self._parse_books(self._doc),
        }
